var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var _InfiniteCanvas_app;
import { html, css, LitElement } from 'lit';
import { Task } from '@lit/task';
import { customElement, property, state } from 'lit/decorators.js';
import { App, DefaultPlugins, Pen } from '@infinite-canvas-tutorial/ecs';
import { setBasePath } from '@shoelace-style/shoelace/dist/utilities/base-path.js';
import '@shoelace-style/shoelace/dist/components/spinner/spinner.js';
import '@shoelace-style/shoelace/dist/components/icon/icon.js';
import '@shoelace-style/shoelace/dist/components/resize-observer/resize-observer.js';
import '@shoelace-style/shoelace/dist/components/alert/alert.js';
/**
 * @see https://shoelace.style/getting-started/themes
 */
import '@shoelace-style/shoelace/dist/themes/light.css';
import '@shoelace-style/shoelace/dist/themes/dark.css';
import { UIPlugin } from '../plugins';
import { Event } from '../event';
import { checkWebGPUSupport } from '../utils';
let InfiniteCanvas = class InfiniteCanvas extends LitElement {
    constructor() {
        super(...arguments);
        this.renderer = 'webgl';
        this.theme = 'light';
        this.shaderCompilerPath = 'https://unpkg.com/@antv/g-device-api@1.6.8/dist/pkg/glsl_wgsl_compiler_bg.wasm';
        this.shoelaceBasePath = 'https://cdn.jsdelivr.net/npm/@shoelace-style/shoelace@2.19.1/cdn';
        this.zoom = 1;
        this.pen = Pen.HAND;
        this.pens = [Pen.HAND, Pen.SELECT, Pen.DRAW_RECT];
        _InfiniteCanvas_app.set(this, void 0);
        this.initCanvas = new Task(this, {
            task: (_a) => __awaiter(this, [_a], void 0, function* ([renderer, shaderCompilerPath]) {
                if (renderer === 'webgpu') {
                    yield checkWebGPUSupport();
                }
                setBasePath(this.shoelaceBasePath);
                const zoom = this.zoom;
                this.addEventListener(Event.ZOOM_CHANGED, (e) => {
                    this.zoom = e.detail.zoom;
                });
                const pen = this.pen;
                this.addEventListener(Event.PEN_CHANGED, (e) => {
                    this.pen = e.detail.pen;
                });
                const canvas = document.createElement('canvas');
                __classPrivateFieldSet(this, _InfiniteCanvas_app, new App().addPlugins(...DefaultPlugins, [
                    UIPlugin,
                    {
                        container: this,
                        canvas,
                        renderer,
                        shaderCompilerPath,
                        zoom,
                        pen,
                    },
                ]), "f");
                this.dispatchEvent(new CustomEvent(Event.READY, { detail: __classPrivateFieldGet(this, _InfiniteCanvas_app, "f") }));
                yield __classPrivateFieldGet(this, _InfiniteCanvas_app, "f").run();
                return canvas;
            }),
            args: () => [this.renderer, this.shaderCompilerPath],
        });
    }
    connectedCallback() {
        super.connectedCallback();
        this.addEventListener('sl-resize', this.resize);
    }
    disconnectedCallback() {
        this.removeEventListener('sl-resize', this.resize);
        if (__classPrivateFieldGet(this, _InfiniteCanvas_app, "f")) {
            __classPrivateFieldGet(this, _InfiniteCanvas_app, "f").exit();
        }
        super.disconnectedCallback();
    }
    resize(event) {
        var _a;
        const detail = event.detail;
        const { width, height } = detail.entries[0].contentRect;
        const dpr = window.devicePixelRatio;
        if (width && height) {
            const $canvas = (_a = this.shadowRoot) === null || _a === void 0 ? void 0 : _a.querySelector('canvas');
            $canvas.width = width * dpr;
            $canvas.height = height * dpr;
            this.dispatchEvent(new CustomEvent(Event.RESIZED, {
                detail: { width, height },
                bubbles: true,
                composed: true,
            }));
        }
    }
    render() {
        return this.initCanvas.render({
            pending: () => html `<sl-spinner></sl-spinner>`,
            complete: ($canvas) => html `
        <sl-resize-observer>
          ${$canvas}
          <ic-shoelace-zoom-toolbar
            zoom=${this.zoom}
          ></ic-shoelace-zoom-toolbar>
          <ic-shoelace-pen-toolbar
            pen=${this.pen}
            pens=${JSON.stringify(this.pens)}
          ></ic-shoelace-pen-toolbar>
          <ic-shoelace-exporter></ic-shoelace-exporter>
        </sl-resize-observer>
      `,
            error: (e) => {
                console.error(e);
                return html `<sl-alert variant="danger" open>
          <sl-icon slot="icon" name="exclamation-octagon"></sl-icon>
          <strong>Initialize canvas failed</strong><br />
          ${e.message}
        </sl-alert>`;
            },
        });
    }
};
_InfiniteCanvas_app = new WeakMap();
InfiniteCanvas.styles = css `
    :host {
      display: block;
      position: relative;
    }

    :host ic-shoelace-dark-mode {
      position: absolute;
      right: 52px;
      top: 16px;
    }

    :host ic-shoelace-exporter {
      position: absolute;
      right: 16px;
      top: 16px;
    }

    :host ic-shoelace-pen-toolbar {
      position: absolute;
      left: 50%;
      top: 16px;
      transform: translateX(-50%);
    }

    :host ic-shoelace-zoom-toolbar {
      position: absolute;
      right: 16px;
      bottom: 16px;
    }

    canvas {
      width: 100%;
      height: 100%;
      outline: none;
      padding: 0;
      margin: 0;
      touch-action: none;
    }
  `;
__decorate([
    property()
], InfiniteCanvas.prototype, "renderer", void 0);
__decorate([
    property()
], InfiniteCanvas.prototype, "theme", void 0);
__decorate([
    property({
        attribute: 'shader-compiler-path',
    })
], InfiniteCanvas.prototype, "shaderCompilerPath", void 0);
__decorate([
    property({
        attribute: 'shoelace-base-path',
    })
], InfiniteCanvas.prototype, "shoelaceBasePath", void 0);
__decorate([
    property({ type: Number }),
    state()
], InfiniteCanvas.prototype, "zoom", void 0);
__decorate([
    property(),
    state()
], InfiniteCanvas.prototype, "pen", void 0);
__decorate([
    property({ type: Array })
], InfiniteCanvas.prototype, "pens", void 0);
InfiniteCanvas = __decorate([
    customElement('ic-shoelace-canvas')
], InfiniteCanvas);
export { InfiniteCanvas };
//# sourceMappingURL=infinite-canvas.js.map