var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _InfiniteCanvas_provider;
import { html, css, LitElement } from 'lit';
import { Task } from '@lit/task';
import { ContextProvider } from '@lit/context';
import { customElement, property } from 'lit/decorators.js';
import { Camera, Canvas, Commands, Pen, PreStartUp, system, System, ThemeMode, Transform, } from '@infinite-canvas-tutorial/ecs';
import { Task as TaskEnum, appStateContext } from '../context';
import { Event } from '../event';
import { checkWebGPUSupport } from '../utils';
import '@spectrum-web-components/theme/sp-theme.js';
import '@spectrum-web-components/theme/src/themes.js';
import '@spectrum-web-components/alert-banner/sp-alert-banner.js';
import '@spectrum-web-components/progress-circle/sp-progress-circle.js';
import { ZoomLevelSystem } from '../systems';
import { Container } from '../components';
let initCanvasSystemCounter = 0;
const initCanvasSystemClasses = [];
const TOP_NAVBAR_HEIGHT = 48;
let InfiniteCanvas = class InfiniteCanvas extends LitElement {
    constructor() {
        super(...arguments);
        this.renderer = 'webgl';
        this.shaderCompilerPath = 'https://unpkg.com/@antv/g-device-api@1.6.8/dist/pkg/glsl_wgsl_compiler_bg.wasm';
        this.appState = {
            theme: {
                mode: ThemeMode.LIGHT,
                colors: {
                    [ThemeMode.LIGHT]: {},
                    [ThemeMode.DARK]: {},
                },
            },
            camera: {
                zoom: 1,
            },
            penbar: {
                all: [Pen.HAND, Pen.SELECT, Pen.DRAW_RECT],
                selected: [Pen.HAND],
            },
            taskbar: {
                all: [TaskEnum.SHOW_LAYERS_PANEL, TaskEnum.SHOW_PROPERTIES_PANEL],
                selected: [],
            },
        };
        _InfiniteCanvas_provider.set(this, new ContextProvider(this, { context: appStateContext }));
        this.initCanvas = new Task(this, {
            task: (_a) => __awaiter(this, [_a], void 0, function* ([renderer, shaderCompilerPath]) {
                if (renderer === 'webgpu') {
                    yield checkWebGPUSupport();
                }
                __classPrivateFieldGet(this, _InfiniteCanvas_provider, "f").setValue(this.appState);
                const { width, height } = this.getBoundingClientRect();
                this.addEventListener(Event.ZOOM_CHANGED, (e) => {
                    __classPrivateFieldGet(this, _InfiniteCanvas_provider, "f").setValue(Object.assign(Object.assign({}, __classPrivateFieldGet(this, _InfiniteCanvas_provider, "f").value), { camera: Object.assign(Object.assign({}, __classPrivateFieldGet(this, _InfiniteCanvas_provider, "f").value.camera), { zoom: e.detail.zoom }) }));
                });
                this.addEventListener(Event.PEN_CHANGED, (e) => {
                    __classPrivateFieldGet(this, _InfiniteCanvas_provider, "f").setValue(Object.assign(Object.assign({}, __classPrivateFieldGet(this, _InfiniteCanvas_provider, "f").value), { penbar: Object.assign(Object.assign({}, __classPrivateFieldGet(this, _InfiniteCanvas_provider, "f").value.penbar), { selected: e.detail.selected }) }));
                });
                this.addEventListener(Event.TASK_CHANGED, (e) => {
                    __classPrivateFieldGet(this, _InfiniteCanvas_provider, "f").setValue(Object.assign(Object.assign({}, __classPrivateFieldGet(this, _InfiniteCanvas_provider, "f").value), { taskbar: Object.assign(Object.assign({}, __classPrivateFieldGet(this, _InfiniteCanvas_provider, "f").value.taskbar), { selected: e.detail.selected }) }));
                });
                const $canvas = document.createElement('canvas');
                $canvas.style.width = '100%';
                $canvas.style.height = 'calc(100% - 48px)';
                const { camera: { zoom }, } = this.appState;
                // eslint-disable-next-line @typescript-eslint/no-this-alias
                const self = this;
                class InitCanvasSystem extends System {
                    constructor() {
                        super();
                        this.commands = new Commands(this);
                        this.query((q) => q.using(Container, Canvas, Camera, Transform).write);
                    }
                    initialize() {
                        const canvas = this.commands
                            .spawn(new Canvas({
                            element: $canvas,
                            width,
                            height: height - TOP_NAVBAR_HEIGHT, // TODO: remove hardcoded top navbar height
                            devicePixelRatio: window.devicePixelRatio,
                            renderer,
                            shaderCompilerPath,
                        }))
                            .id();
                        canvas.add(Container, { element: self });
                        this.canvasEntity = canvas.hold();
                        this.commands.spawn(new Camera({
                            canvas: this.canvasEntity,
                        }), new Transform({
                            scale: {
                                x: 1 / zoom,
                                y: 1 / zoom,
                            },
                        }));
                        this.commands.execute();
                        self.addEventListener(Event.RESIZED, (e) => {
                            const { width, height } = e.detail;
                            Object.assign(this.canvasEntity.write(Canvas), {
                                width,
                                height,
                            });
                        });
                        self.addEventListener(Event.PEN_CHANGED, (e) => {
                            const { selected } = e.detail;
                            Object.assign(this.canvasEntity.write(Canvas), {
                                pen: selected[0],
                            });
                        });
                        self.addEventListener(Event.DESTROY, (e) => {
                            this.canvasEntity.delete();
                        });
                    }
                }
                Object.defineProperty(InitCanvasSystem, 'name', {
                    value: `InitCanvasSystem${initCanvasSystemCounter++}`,
                });
                if (initCanvasSystemClasses.length > 0) {
                    system((s) => s
                        .before(PreStartUp)
                        .beforeWritersOf(Canvas)
                        .after(...initCanvasSystemClasses))(InitCanvasSystem);
                }
                else {
                    system((s) => s.after(PreStartUp).before(ZoomLevelSystem).beforeWritersOf(Canvas))(InitCanvasSystem);
                }
                initCanvasSystemClasses.push(InitCanvasSystem);
                this.dispatchEvent(new CustomEvent(Event.READY, { detail: $canvas }));
                return $canvas;
            }),
            args: () => [this.renderer, this.shaderCompilerPath],
        });
    }
    connectedCallback() {
        super.connectedCallback();
        this.resizeObserver = new ResizeObserver((entries) => this.handleResize(entries));
        this.updateComplete.then(() => this.resizeObserver.observe(this));
    }
    disconnectedCallback() {
        var _a;
        super.disconnectedCallback();
        (_a = this.resizeObserver) === null || _a === void 0 ? void 0 : _a.unobserve(this);
    }
    handleResize(entries) {
        var _a;
        const { width, height } = entries[0].contentRect;
        const dpr = window.devicePixelRatio;
        if (width && height) {
            const $canvas = (_a = this.shadowRoot) === null || _a === void 0 ? void 0 : _a.querySelector('canvas');
            $canvas.width = width * dpr;
            $canvas.height = (height - TOP_NAVBAR_HEIGHT) * dpr;
            this.dispatchEvent(new CustomEvent(Event.RESIZED, {
                detail: { width, height: height - TOP_NAVBAR_HEIGHT },
                bubbles: true,
                composed: true,
            }));
        }
    }
    render() {
        const themeWrapper = (content) => html `<sp-theme
        system="spectrum"
        color="${this.appState.theme.mode}"
        scale="medium"
        >${typeof content === 'string' ? html `${content}` : content}</sp-theme
      >`;
        return this.initCanvas.render({
            pending: () => themeWrapper(html `<sp-progress-circle
            label="A small representation of an unclear amount of work"
            indeterminate
            size="s"
          ></sp-progress-circle>`),
            complete: ($canvas) => themeWrapper(html `<ic-spectrum-top-navbar></ic-spectrum-top-navbar>
            <ic-spectrum-penbar></ic-spectrum-penbar>
            <ic-spectrum-taskbar></ic-spectrum-taskbar>
            <ic-spectrum-layers-panel></ic-spectrum-layers-panel>
            ${$canvas}`),
            error: (e) => {
                console.error(e);
                return themeWrapper(html `<sp-alert-banner open variant="negative">
            Initialize canvas failed<br />
            <strong></strong><br />
            ${e.message}
          </sp-alert-banner>`);
            },
        });
    }
};
_InfiniteCanvas_provider = new WeakMap();
InfiniteCanvas.styles = css `
    :host {
      display: block;
      position: relative;
    }

    ic-spectrum-top-navbar {
      padding: var(--spectrum-global-dimension-size-100);
      display: flex;
      justify-content: end;
      align-items: center;
    }

    ic-spectrum-penbar {
      position: absolute;
      top: 48px;
      left: 0;
    }

    ic-spectrum-taskbar {
      position: absolute;
      top: 48px;
      right: 0;
    }

    ic-spectrum-layers-panel {
      position: absolute;
      top: 48px;
      right: 54px;
    }

    canvas {
      width: 100%;
      height: 100%;
      outline: none;
      padding: 0;
      margin: 0;
      touch-action: none;
    }
  `;
__decorate([
    property()
], InfiniteCanvas.prototype, "renderer", void 0);
__decorate([
    property({
        attribute: 'shader-compiler-path',
    })
], InfiniteCanvas.prototype, "shaderCompilerPath", void 0);
__decorate([
    property({ type: Object })
], InfiniteCanvas.prototype, "appState", void 0);
InfiniteCanvas = __decorate([
    customElement('ic-spectrum-canvas')
], InfiniteCanvas);
export { InfiniteCanvas };
//# sourceMappingURL=infinite-canvas.js.map