"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _InfiniteCanvas_provider;
Object.defineProperty(exports, "__esModule", { value: true });
exports.InfiniteCanvas = void 0;
const lit_1 = require("lit");
const task_1 = require("@lit/task");
const context_1 = require("@lit/context");
const decorators_js_1 = require("lit/decorators.js");
const ecs_1 = require("@infinite-canvas-tutorial/ecs");
const context_2 = require("../context");
const event_1 = require("../event");
const utils_1 = require("../utils");
require("@spectrum-web-components/theme/sp-theme.js");
require("@spectrum-web-components/theme/src/themes.js");
require("@spectrum-web-components/alert-banner/sp-alert-banner.js");
require("@spectrum-web-components/progress-circle/sp-progress-circle.js");
const systems_1 = require("../systems");
const components_1 = require("../components");
let initCanvasSystemCounter = 0;
const initCanvasSystemClasses = [];
const TOP_NAVBAR_HEIGHT = 48;
let InfiniteCanvas = class InfiniteCanvas extends lit_1.LitElement {
    constructor() {
        super(...arguments);
        this.renderer = 'webgl';
        this.shaderCompilerPath = 'https://unpkg.com/@antv/g-device-api@1.6.8/dist/pkg/glsl_wgsl_compiler_bg.wasm';
        this.appState = {
            theme: {
                mode: ecs_1.ThemeMode.LIGHT,
                colors: {
                    [ecs_1.ThemeMode.LIGHT]: {},
                    [ecs_1.ThemeMode.DARK]: {},
                },
            },
            camera: {
                zoom: 1,
            },
            penbar: {
                all: [ecs_1.Pen.HAND, ecs_1.Pen.SELECT, ecs_1.Pen.DRAW_RECT],
                selected: [ecs_1.Pen.HAND],
            },
            taskbar: {
                all: [context_2.Task.SHOW_LAYERS_PANEL, context_2.Task.SHOW_PROPERTIES_PANEL],
                selected: [],
            },
        };
        _InfiniteCanvas_provider.set(this, new context_1.ContextProvider(this, { context: context_2.appStateContext }));
        this.initCanvas = new task_1.Task(this, {
            task: (_a) => __awaiter(this, [_a], void 0, function* ([renderer, shaderCompilerPath]) {
                if (renderer === 'webgpu') {
                    yield (0, utils_1.checkWebGPUSupport)();
                }
                __classPrivateFieldGet(this, _InfiniteCanvas_provider, "f").setValue(this.appState);
                const { width, height } = this.getBoundingClientRect();
                this.addEventListener(event_1.Event.ZOOM_CHANGED, (e) => {
                    __classPrivateFieldGet(this, _InfiniteCanvas_provider, "f").setValue(Object.assign(Object.assign({}, __classPrivateFieldGet(this, _InfiniteCanvas_provider, "f").value), { camera: Object.assign(Object.assign({}, __classPrivateFieldGet(this, _InfiniteCanvas_provider, "f").value.camera), { zoom: e.detail.zoom }) }));
                });
                this.addEventListener(event_1.Event.PEN_CHANGED, (e) => {
                    __classPrivateFieldGet(this, _InfiniteCanvas_provider, "f").setValue(Object.assign(Object.assign({}, __classPrivateFieldGet(this, _InfiniteCanvas_provider, "f").value), { penbar: Object.assign(Object.assign({}, __classPrivateFieldGet(this, _InfiniteCanvas_provider, "f").value.penbar), { selected: e.detail.selected }) }));
                });
                this.addEventListener(event_1.Event.TASK_CHANGED, (e) => {
                    __classPrivateFieldGet(this, _InfiniteCanvas_provider, "f").setValue(Object.assign(Object.assign({}, __classPrivateFieldGet(this, _InfiniteCanvas_provider, "f").value), { taskbar: Object.assign(Object.assign({}, __classPrivateFieldGet(this, _InfiniteCanvas_provider, "f").value.taskbar), { selected: e.detail.selected }) }));
                });
                const $canvas = document.createElement('canvas');
                $canvas.style.width = '100%';
                $canvas.style.height = 'calc(100% - 48px)';
                const { camera: { zoom }, } = this.appState;
                // eslint-disable-next-line @typescript-eslint/no-this-alias
                const self = this;
                class InitCanvasSystem extends ecs_1.System {
                    constructor() {
                        super();
                        this.commands = new ecs_1.Commands(this);
                        this.query((q) => q.using(components_1.Container, ecs_1.Canvas, ecs_1.Camera, ecs_1.Transform).write);
                    }
                    initialize() {
                        const canvas = this.commands
                            .spawn(new ecs_1.Canvas({
                            element: $canvas,
                            width,
                            height: height - TOP_NAVBAR_HEIGHT, // TODO: remove hardcoded top navbar height
                            devicePixelRatio: window.devicePixelRatio,
                            renderer,
                            shaderCompilerPath,
                        }))
                            .id();
                        canvas.add(components_1.Container, { element: self });
                        this.canvasEntity = canvas.hold();
                        this.commands.spawn(new ecs_1.Camera({
                            canvas: this.canvasEntity,
                        }), new ecs_1.Transform({
                            scale: {
                                x: 1 / zoom,
                                y: 1 / zoom,
                            },
                        }));
                        this.commands.execute();
                        self.addEventListener(event_1.Event.RESIZED, (e) => {
                            const { width, height } = e.detail;
                            Object.assign(this.canvasEntity.write(ecs_1.Canvas), {
                                width,
                                height,
                            });
                        });
                        self.addEventListener(event_1.Event.PEN_CHANGED, (e) => {
                            const { selected } = e.detail;
                            Object.assign(this.canvasEntity.write(ecs_1.Canvas), {
                                pen: selected[0],
                            });
                        });
                        self.addEventListener(event_1.Event.DESTROY, (e) => {
                            this.canvasEntity.delete();
                        });
                    }
                }
                Object.defineProperty(InitCanvasSystem, 'name', {
                    value: `InitCanvasSystem${initCanvasSystemCounter++}`,
                });
                if (initCanvasSystemClasses.length > 0) {
                    (0, ecs_1.system)((s) => s
                        .before(ecs_1.PreStartUp)
                        .beforeWritersOf(ecs_1.Canvas)
                        .after(...initCanvasSystemClasses))(InitCanvasSystem);
                }
                else {
                    (0, ecs_1.system)((s) => s.after(ecs_1.PreStartUp).before(systems_1.ZoomLevelSystem).beforeWritersOf(ecs_1.Canvas))(InitCanvasSystem);
                }
                initCanvasSystemClasses.push(InitCanvasSystem);
                this.dispatchEvent(new CustomEvent(event_1.Event.READY, { detail: $canvas }));
                return $canvas;
            }),
            args: () => [this.renderer, this.shaderCompilerPath],
        });
    }
    connectedCallback() {
        super.connectedCallback();
        this.resizeObserver = new ResizeObserver((entries) => this.handleResize(entries));
        this.updateComplete.then(() => this.resizeObserver.observe(this));
    }
    disconnectedCallback() {
        var _a;
        super.disconnectedCallback();
        (_a = this.resizeObserver) === null || _a === void 0 ? void 0 : _a.unobserve(this);
    }
    handleResize(entries) {
        var _a;
        const { width, height } = entries[0].contentRect;
        const dpr = window.devicePixelRatio;
        if (width && height) {
            const $canvas = (_a = this.shadowRoot) === null || _a === void 0 ? void 0 : _a.querySelector('canvas');
            $canvas.width = width * dpr;
            $canvas.height = (height - TOP_NAVBAR_HEIGHT) * dpr;
            this.dispatchEvent(new CustomEvent(event_1.Event.RESIZED, {
                detail: { width, height: height - TOP_NAVBAR_HEIGHT },
                bubbles: true,
                composed: true,
            }));
        }
    }
    render() {
        const themeWrapper = (content) => (0, lit_1.html) `<sp-theme
        system="spectrum"
        color="${this.appState.theme.mode}"
        scale="medium"
        >${typeof content === 'string' ? (0, lit_1.html) `${content}` : content}</sp-theme
      >`;
        return this.initCanvas.render({
            pending: () => themeWrapper((0, lit_1.html) `<sp-progress-circle
            label="A small representation of an unclear amount of work"
            indeterminate
            size="s"
          ></sp-progress-circle>`),
            complete: ($canvas) => themeWrapper((0, lit_1.html) `<ic-spectrum-top-navbar></ic-spectrum-top-navbar>
            <ic-spectrum-penbar></ic-spectrum-penbar>
            <ic-spectrum-taskbar></ic-spectrum-taskbar>
            <ic-spectrum-layers-panel></ic-spectrum-layers-panel>
            ${$canvas}`),
            error: (e) => {
                console.error(e);
                return themeWrapper((0, lit_1.html) `<sp-alert-banner open variant="negative">
            Initialize canvas failed<br />
            <strong></strong><br />
            ${e.message}
          </sp-alert-banner>`);
            },
        });
    }
};
exports.InfiniteCanvas = InfiniteCanvas;
_InfiniteCanvas_provider = new WeakMap();
InfiniteCanvas.styles = (0, lit_1.css) `
    :host {
      display: block;
      position: relative;
    }

    ic-spectrum-top-navbar {
      padding: var(--spectrum-global-dimension-size-100);
      display: flex;
      justify-content: end;
      align-items: center;
    }

    ic-spectrum-penbar {
      position: absolute;
      top: 48px;
      left: 0;
    }

    ic-spectrum-taskbar {
      position: absolute;
      top: 48px;
      right: 0;
    }

    ic-spectrum-layers-panel {
      position: absolute;
      top: 48px;
      right: 54px;
    }

    canvas {
      width: 100%;
      height: 100%;
      outline: none;
      padding: 0;
      margin: 0;
      touch-action: none;
    }
  `;
__decorate([
    (0, decorators_js_1.property)()
], InfiniteCanvas.prototype, "renderer", void 0);
__decorate([
    (0, decorators_js_1.property)({
        attribute: 'shader-compiler-path',
    })
], InfiniteCanvas.prototype, "shaderCompilerPath", void 0);
__decorate([
    (0, decorators_js_1.property)({ type: Object })
], InfiniteCanvas.prototype, "appState", void 0);
exports.InfiniteCanvas = InfiniteCanvas = __decorate([
    (0, decorators_js_1.customElement)('ic-spectrum-canvas')
], InfiniteCanvas);
//# sourceMappingURL=infinite-canvas.js.map